include "Zeropage.inc"
include "Vram.inc"
include "Chars.inc"

ext CharColors_, SpritePattern_, EnemyPattern_
ext EnemyRowLeft_, EnemyChar_
ext StarDotOffset_, StarChar_

SpritePatternRam equ VicRam+SpriteSize*Pattern_Enemy
EnemyPatternRam equ PatternRam+CharHeight*Char_Enemy
StarPatternRam equ PatternRam+CharHeight*Char_Star

pDestination equ ZW0
pSource equ ZW1
pColorMap equ ZW2
wBits equ ZW2

count equ ZB0
yCount equ ZB1
destinationOffset equ ZB2
flagOffset equ ZB2
sourceOffset equ ZB3
flagBit equ ZB3
lShift equ ZB1
rShift equ ZB4
SetRowFlags_low equ ZB0
SetRowFlags_high equ ZB1


dseg
Background_: public Background_
    defs WindowWidth*WindowHeight


; void InitCharColors()
cseg
ColorSource:
defb 64,1
defb 16,3
defb 6,1
defb 16,4
defb 16,10
defb 16,7
defb 0

InitCharColors_: public InitCharColors_
    pha
        ldx #0
        ldy #0
        do
            lda ColorSource,x
        while ne
            sta <count
            inx
            lda ColorSource,x
            inx
            do
                sta CharColors_,y
                iny
                dec <count
            while ne | wend
        wend
    pla
rts


; void InitSpritePatterns();
cseg
InitSpritePatterns_: public InitSpritePatterns_
    pha
        lda #low SpritePattern_ | sta <pSource
        lda #high SpritePattern_ | sta <pSource+1

        lda #low SpritePatternRam | sta <pDestination
        lda #high SpritePatternRam | sta <pDestination+1

        lda #Pattern_End-Pattern_Enemy | sta <count
        do
            ldy #0 | sty <sourceOffset | sty <destinationOffset
            lda #CharHeight*2 | sta <yCount
            do
                ldx #2
                do
                    ldy <sourceOffset | lda [<pSource],y | iny | sty <sourceOffset
                    ldy <destinationOffset | sta [<pDestination],y | iny | sty <destinationOffset
                    dex
                while ne | wend
                lda #0
                sta [<pDestination],y | iny | sty <destinationOffset
                dec <yCount
            while ne | wend
            ldy <destinationOffset
            lda #0
            do
                sta [<pDestination],y | iny
                cpy #SpriteSize
            while ne | wend

            lda <pSource | clc|adc #CharHeight*4 | sta <pSource
            lda <pSource+1 | adc #0 | sta <pSource+1

            lda <pDestination | clc|adc #SpriteSize | sta <pDestination
            lda <pDestination+1 | adc #0 | sta <pDestination+1

            dec <count
        while ne | wend
    pla
rts


; ptr<byte> Put(ptr<byte> pVram, byte c);
dseg
Put_@Param0: public Put_@Param0
Put_pVram:
    defw 0
Put_@Param1: public Put_@Param1
Put_c:
    defb 0
cseg
Put_: public Put_
    pha
        lda Put_pVram
        sta <ZW0
        lda Put_pVram+1
        sta <ZW0+1
        lda Put_c
        ldy #0
        sta (<ZW0),y
        tax
        lda <ZW0+1
        clc|adc #high(ColorMap-Vram)
        sta <ZW0+1
        lda CharColors_,x
        sta (<ZW0),y
        
        lda Put_pVram | clc|adc #1 | tay
        lda Put_pVram+1 | adc #0 | tax
    pla
rts


; void ClearBackground();
cseg
ClearBackground_: public ClearBackground_
    pha
        lda #low Background_ | sta <pDestination
        lda #high Background_ | sta <pDestination+1

        ldx #WindowHeight
        do
            lda #Char_Space
            ldy #0
            do
                sta (<pDestination),y
                iny
                cpy #WindowWidth
            while ne | wend    
            lda <pDestination | clc|adc #WindowWidth | sta <pDestination
            lda <pDestination+1 | adc #0 | sta <pDestination+1

            dex
        while ne | wend
    pla
rts


; void BackgroundToVram();
cseg
BackgroundToVram_: public BackgroundToVram_
    pha
        ldx #0 | stx <flagOffset
        lda #1 | sta <flagBit

        lda #low Background_ | sta <pSource
        lda #high Background_ | sta <pSource+1

        lda #low Vram | sta <pDestination
        lda #high Vram | sta <pDestination+1

        lda #low ColorMap | sta <pColorMap
        lda #high ColorMap | sta <pColorMap+1

        lda #WindowHeight | sta <count
        do
            ldy #0
            do
                lda (<pSource),y | sta (<pDestination),y
                tax | lda CharColors_,x | sta (<pColorMap),y
                iny
                cpy #WindowWidth
            while ne | wend

            lda <pSource | clc|adc #WindowWidth | sta <pSource
            lda <pSource+1 | adc #0 | sta <pSource+1

            lda <pDestination | clc|adc #VramWidth | sta <pDestination
            lda <pDestination+1 | adc #0 | sta <pDestination+1

            lda <pColorMap | clc|adc #VramWidth | sta <pColorMap
            lda <pColorMap+1 | adc #0 | sta <pColorMap+1

            dec <count
        while ne | wend
    pla
rts


; ptr<byte> BackgroundPtr(byte x, byte y);
dseg
BackgroundPtr_@Param0: public BackgroundPtr_@Param0
BackgroundPtr_x:
    defb 0
BackgroundPtr_@Param1: public BackgroundPtr_@Param1
BackgroundPtr_y:
    defb 0
cseg
BackgroundPtr_: public BackgroundPtr_
    pha
        lda BackgroundPtr_y
        asl a ;*2
        asl a ;*4
        asl a ;*8
        sta <pDestination
        lda #0 | sta <pDestination+1
        asl <pDestination | rol <pDestination+1 ;*16
        asl <pDestination | rol <pDestination+1 ;*32
        lda <pDestination | clc|adc BackgroundPtr_x | sta <pDestination
        lda <pDestination+1 | adc #0 | sta <pDestination+1
        lda <pDestination | clc|adc #low Background_ | tay
        lda <pDestination+1 | adc #high Background_ | tax
    pla
rts


; void UpdateEnemyChars();
cseg
UpdateEnemyChars_: public UpdateEnemyChars_
    pha
        lda EnemyChar_
        sec|sbc #Char_Enemy
        asl a | asl a | asl a
        clc|adc #low EnemyPatternRam | sta <pDestination
        lda #high EnemyPatternRam | adc #0 | sta <pDestination+1

        lda #0 | sta <destinationOffset

        lda #low EnemyPattern_ | sta <pSource
        lda #high EnemyPattern_ | sta <pSource+1

        lda EnemyRowLeft_ | and #7 | sta <rShift
        lda #8 | sec|sbc <rShift | sta <lShift

        lda #3 | sta <count
        do
            ldy #0
            do
                lda (<pSource),y | iny
                iny
                sty <sourceOffset
                
                ldx <rShift
                do
                while ne
                    lsr a
                    dex
                wend
                ldy <destinationOffset
                sta (pDestination),y
                iny | sty <destinationOffset

                ldy <sourceOffset
                cpy #CharHeight*2*2
            while ne | wend

            ldy #0
            do
                iny
                lda (<pSource),y | iny
                sty <sourceOffset
                
                ldx <lShift
                do
                while ne
                    asl a
                    dex
                wend
                ldy <destinationOffset
                sta (pDestination),y
                iny | sty <destinationOffset

                ldy <sourceOffset
                cpy #CharHeight*2*2
            while ne | wend

            ldy #0
            do
                lda (<pSource),y | sta <wBits | iny
                lda (<pSource),y | sta <wBits+1 | iny
                sty <sourceOffset

                ldx <rShift
                do
                while ne
                    lsr <wBits+1 | ror <wBits
                    dex
                wend
                lda <wBits
                ldy <destinationOffset
                sta (pDestination),y
                iny | sty <destinationOffset

                ldy <sourceOffset
                cpy #CharHeight*2*2
            while ne | wend

            ldy #0
            do
                lda (<pSource),y | sta <wBits | iny
                lda (<pSource),y | sta <wBits+1 | iny
                sty <sourceOffset

                ldx <lShift
                do
                while ne
                    asl <wBits+1 | rol <wBits
                    dex
                wend
                lda <wBits
                ldy <destinationOffset
                sta (pDestination),y
                iny | sty <destinationOffset

                ldy <sourceOffset
                cpy #CharHeight*2*2
            while ne | wend

            lda <pSource | clc|adc #CharHeight*4 | sta <pSource
            lda <pSource+1 | adc #0 | sta <pSource+1

            lda <pDestination | clc|adc #CharHeight*2*4 | sta <pDestination
            lda <pDestination+1 | adc #0 | sta <pDestination+1

            dec <count
        while ne | wend
    pla
rts


; void UpdateStarChars();
cseg
UpdateStarChars_: public UpdateStarChars_
    pha
        lda #low StarPatternRam

        lda StarDotOffset_
        and #7  | sta <count
        
        lda StarChar_
        cmp #Char_Star
        if eq
            ldx #0
        else
            ldx #CharHeight
        endif

        lda #0
        ldy <count
        do
        while ne
            sta StarPatternRam,x | inx
            dey
        wend

        lda #$10
        sta StarPatternRam,x | inx

        lda #0
        ldy <count
        do
            iny
            cpy #8
        while ne
            sta StarPatternRam,x | inx
        wend
    pla
rts